# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Script to reinitialize surveil."""

import optparse
import subprocess
import sys

import pymongo
import surveilclient.client as sc

from surveil.api import config


def main():
    parser = optparse.OptionParser()
    parser.add_option('-d', '--demo',
                      default=False,
                      dest='demo',
                      action="store_true")
    opts, _ = parser.parse_args(sys.argv)

    # Create a basic config in mongodb
    mongo = pymongo.MongoClient(config.surveil_api_config['mongodb_uri'])

    if opts.demo is True:
        # Drop the current shinken config
        mongo.drop_database('shinken')

    if mongo.surveil.init.count() == 0:
        # Mark packs as uploaded
        print("Uploading packs...")
        mongo.surveil.init.insert({"source": "surveil-init script"})

        # Load the shinken packs
        subprocess.call(
            [
                "surveil-pack-upload",
                "--mongo-url=mongo",
                "--mongo-port=27017",
                "/usr/share/monitoring/packs/sfl/openstack-keystone/",
            ]
        )

        subprocess.call(
            [
                "surveil-pack-upload",
                "--mongo-url=mongo",
                "--mongo-port=27017",
                "/usr/share/monitoring/packs/sfl/openstack-glance/",
            ]
        )

        subprocess.call(
            [
                "surveil-pack-upload",
                "--mongo-url=mongo",
                "--mongo-port=27017",
                "/usr/share/monitoring/packs/sfl/generic-host/",
            ]
        )

        subprocess.call(
            [
                "surveil-pack-upload",
                "--mongo-url=mongo",
                "--mongo-port=27017",
                "/usr/share/monitoring/packs/sfl/openstack-nova/",
            ]
        )

        subprocess.call(
            [
                "surveil-pack-upload",
                "--mongo-url=mongo",
                "--mongo-port=27017",
                "/usr/share/monitoring/packs/sfl/openstack-cinder/",
            ]
        )

        subprocess.call(
            [
                "surveil-pack-upload",
                "--mongo-url=mongo",
                "--mongo-port=27017",
                "/usr/share/monitoring/packs/sfl/openstack-host/",
            ]
        )

    else:
        print("Skipping pack upload...")

    cli_surveil = sc.Client('http://localhost:8080/v2',
                            auth_url='http://localhost:8080/v2/auth',
                            version='2_0')

    # if --demo is specified, you get more hosts.
    if opts.demo is True:

        # shinken's ws-arbiter
        cli_surveil.config.hosts.create(
            use="generic-host",
            contact_groups="admins",
            host_name="ws-arbiter",
            address="localhost"
        )
        cli_surveil.config.services.create(
            check_command="check_tcp!7760",
            check_interval="5",
            check_period="24x7",
            contact_groups="admins",
            contacts="admin",
            host_name="ws-arbiter",
            max_check_attempts="5",
            notification_interval="30",
            notification_period="24x7",
            retry_interval="3",
            service_description="check-ws-arbiter"
        )

        # Linux-keystone template
        cli_surveil.config.hosts.create(
            host_name='test_keystone',
            use='linux-keystone',
            address='127.0.0.1',
            custom_fields={
                "_OS_AUTH_URL": "bla",
                "_OS_USERNAME": "bli",
                "_OS_PASSWORD": "blo",
                "_OS_TENANT":   "blu",
                "_KS_SERVICES": "bly",
                "parents": "localhost"
            }
        )

        # Linux-openstackceilometer template
        cli_surveil.config.hosts.create(
            host_name='openstackceilometer-host',
            use='linux-openstackceilometer',
            address='127.0.0.1',
            custom_fields={
                "_OS_AUTH_URL": "bla",
                "_OS_USERNAME": "bli",
                "_OS_PASSWORD": "blo",
                "_OS_TENANT_NAME":   "blu",
                "_KS_SERVICES": "bly",
                "parents": "localhost"
            }
        )

        # DOWN HOST (cant resolve)
        cli_surveil.config.hosts.create(
            host_name='srv-apache-01',
            use='linux-system-nrpe',
            address='srv-apache-01',
            custom_fields={
                "_TRAFFICLIMIT": "100000",
            }
        )

        # DOWN, and parent down (Network outage)
        cli_surveil.config.hosts.create(
            host_name='myparentisdown',
            address='dfgsdgsdgf',
            parents='srv-apache-01',
        )

        # UP host, no template
        cli_surveil.config.hosts.create(
            host_name='google.com',
            address='google.com'
        )

        # NRPE host, UP
        cli_surveil.config.hosts.create(
            host_name='srv-monitoring-01',
            use='linux-system-nrpe',
            address='127.0.0.1',
            custom_fields={
                "_TRAFFICLIMIT": "500000",
            }
        )

        # Has parent, UP
        cli_surveil.config.hosts.create(
            host_name='sw-iwebcore-01',
            parents='srv-monitoring-01',
            use='generic-host',
            address='127.0.0.1',
            custom_fields={
                "_TRAFFICLIMIT": "200000",
            }
        )

        # Has chain of 2 parents, UP
        cli_surveil.config.hosts.create(
            host_name='srv-ldap-01',
            parents='sw-iwebcore-01',
            use='generic-host',
            address='127.0.0.1',
            custom_fields={
                "_TRAFFICLIMIT": "5000000",
            }
        )

        # UP host with down service
        cli_surveil.config.hosts.create(
            use="generic-host",
            contact_groups="admins",
            host_name="myserviceisdown",
            address="localhost"
        )
        cli_surveil.config.services.create(
            check_command="check_tcp!4553",
            check_interval="5",
            check_period="24x7",
            contact_groups="admins",
            contacts="admin",
            host_name="myserviceisdown",
            max_check_attempts="5",
            notification_interval="30",
            notification_period="24x7",
            retry_interval="3",
            service_description="iamadownservice"
        )

    # Reload the shinken config
    cli_surveil.config.reload_config()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import inspect

import wsme
from wsme import types as wtypes


class Base(wtypes.Base):
    """Base class for all API types."""

    def as_dict(self):
        keys = [
            member[0] for member
            in inspect.getmembers(self.__class__)
            if member[0][0] is not '_' and type(member[1]) is wtypes.wsattr
        ]
        return self.as_dict_from_keys(keys)

    def as_dict_from_keys(self, keys):
        return dict((k, getattr(self, k))
                    for k in keys
                    if hasattr(self, k) and
                    getattr(self, k) != wsme.Unset)



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class MacroModulation(types.Base):
    macromodulation_name = wsme.wsattr(wtypes.text, mandatory=True)
    modulation_period = wsme.wsattr(wtypes.text, mandatory=True)
    # _CRITICAL = wsme.wsattr(int, mandatory=True)
    # _WARNING = wsme.wsattr(int, mandatory=True)

    macros = wsme.wsattr(
        wtypes.DictType(wtypes.text, int),
        mandatory=False
    )

    def __init__(self, **kwargs):
        super(MacroModulation, self).__init__(**kwargs)

        # Custom fields start with '_'. Detect them and assign them.
        macros = [i for i in kwargs.items()
                  if isinstance(i[0], str) and i[0].startswith('_')]
        if len(macros) > 0:
            self.macros = {}
            for item in macros:
                self.macros[item[0]] = item[1]

    def as_dict(self):
        mod_dict = super(MacroModulation, self).as_dict()
        macros = mod_dict.pop("macros", None)
        if macros:
            for item in macros.items():
                mod_dict[item[0]] = item[1]
        return mod_dict

    @classmethod
    def sample(cls):
        return cls(
            macromodulation_name='HighDuringNight',
            modulation_period='night',
            macros={"_CRITICAL": 20, "_WARNING": 10}
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import print_function

import os
import time
import unittest

from surveil.tests import base

from compose.cli import docker_client
from compose import config as compose_config
from compose import project as compose_project
from surveilclient import client as sclient


@unittest.skipIf(os.environ.get('SURVEIL_INTEGRATION_TESTS', None) != 'True',
                 'Skipping integraiton tests')
class MergedIntegrationTest(base.BaseTestCase):

    @classmethod
    def setUpClass(cls):
        surveil_dir = os.path.realpath(
            os.path.join(
                os.path.dirname(os.path.realpath(__file__)),
                "../../../")
        )

        compose_file = os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            'integration.yml'
        )

        project_config = compose_config.from_dictionary(
            compose_config.load_yaml(compose_file),
            working_dir=surveil_dir,
            filename=compose_file
        )

        cls.project = compose_project.Project.from_dicts(
            "surveilintegrationtest",
            project_config,
            docker_client.docker_client()
        )
        cls.project.kill()
        cls.project.remove_stopped()
        cls.project.build()
        cls.project.up()

        cls.client = sclient.Client(
            'http://localhost:8999/v2',
            auth_url='http://localhost:8999/v2/auth',
            version='2_0'
        )

        #  Wait until Surveil is available
        now = time.time()
        while True:
            print("Waiting for surveil... %s" % int(time.time() - now))
            if time.time() < (now + 180):
                try:
                    #  If 'ws-arbiter' is found, Surveil is ready!
                    configured_hosts = cls.client.status.hosts.list()
                    host_found = False
                    for host in configured_hosts:
                        if host['host_name'].decode() == 'ws-arbiter':
                            host_found = True
                            break
                    if host_found:
                        break

                except Exception:
                    pass
                time.sleep(10)
            else:
                raise Exception("Surveil could not start")

    @classmethod
    def tearDownClass(cls):
        cls.project.kill()
        cls.project.remove_stopped()


class SeparatedIntegrationTests(MergedIntegrationTest):

    def setUp(self):
        SeparatedIntegrationTests.setUpClass()

    def tearDown(self):
        SeparatedIntegrationTests.tearDownClass()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.status import live_query
from surveil.api.datamodel.status import live_service
from surveil.api.handlers.status import live_service_handler
from surveil.common import util


class ServicesController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_service.LiveService])
    def get_all(self):
        """Returns all services."""
        handler = live_service_handler.ServiceHandler(pecan.request)
        services = handler.get_all()
        return services

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_service.LiveService], body=live_query.LiveQuery)
    def post(self, query):
        """Given a LiveQuery, returns all matching services."""
        handler = live_service_handler.ServiceHandler(pecan.request)
        services = handler.get_all(live_query=query)
        return services



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from surveil.api.datamodel.status import live_query
from surveil.api.handlers.status import influxdb_query
from surveil.tests import base


class LiveQueryFilterTest(base.BaseTestCase):

    def test_build_where_clause(self):
        filters = {
            "is": {
                "state": [0],
                "description": ["test_keystone"]
            }
        }

        result = influxdb_query._build_where_clause(
            filters
        )

        expected = "WHERE state=0 AND description='test_keystone'"

        self.assert_count_equal_backport(result, expected)

    def test_build_where_clause_no_filters(self):
        filters = {}

        result = influxdb_query._build_where_clause(
            filters
        )

        expected = ""

        self.assert_count_equal_backport(result, expected)

    def test_build_influx_query(self):
        query = live_query.LiveQuery(
            fields=['host_name', 'last_check'],
            filters=json.dumps({}),
        )
        measurement = 'ALERT'
        group_by = ['*', 'host_name']
        limit = 10

        result = influxdb_query.build_influxdb_query(query,
                                                     measurement,
                                                     group_by=group_by,
                                                     limit=limit)

        expected = "SELECT * FROM ALERT GROUP BY *, host_name LIMIT 10"

        self.assert_count_equal_backport(result, expected)

    def test_build_influx_query_orderby(self):
        query = live_query.LiveQuery(
            fields=['host_name', 'last_check'],
            filters=json.dumps({}),
        )
        measurement = 'ALERT'
        group_by = ['*', 'host_name']
        order_by = ['time DESC']
        limit = 10

        result = influxdb_query.build_influxdb_query(query,
                                                     measurement,
                                                     group_by=group_by,
                                                     order_by=order_by,
                                                     limit=limit)

        expected = ("SELECT * FROM ALERT "
                    "GROUP BY *, host_name "
                    "ORDER BY time DESC LIMIT 10")

        self.assert_count_equal_backport(result, expected)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class Service(types.Base):
    host_name = wsme.wsattr(wtypes.text, mandatory=True)

    service_description = wsme.wsattr(wtypes.text, mandatory=True)

    check_command = wsme.wsattr(wtypes.text, mandatory=False)

    max_check_attempts = wsme.wsattr(int, mandatory=False)

    check_interval = wsme.wsattr(int, mandatory=False)

    retry_interval = wsme.wsattr(int, mandatory=False)

    check_period = wsme.wsattr(wtypes.text, mandatory=False)

    notification_interval = wsme.wsattr(int, mandatory=False)

    notification_period = wsme.wsattr(wtypes.text, mandatory=False)

    contacts = wsme.wsattr(wtypes.text, mandatory=False)

    contact_groups = wsme.wsattr(wtypes.text, mandatory=False)

    passive_checks_enabled = wsme.wsattr(wtypes.text, mandatory=False)

    @classmethod
    def sample(cls):
        return cls(
            host_name="sample-server",
            service_description="check-disk-sdb",
            check_command="check-disk!/dev/sdb1",
            max_check_attempts=5,
            check_interval=5,
            retry_interval=3,
            check_period="24x7",
            notification_interval=3,
            notification_period="24x7",
            contacts="surveil-ptl,surveil-bob",
            contact_groups="linux-admins",
            passive_checks_enabled='1',
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import contact
from surveil.api.handlers import handler


class ContactHandler(handler.Handler):
    """Fulfills a request on the contact resource."""

    def get(self, contact_name):
        """Return a contact."""

        c = self.request.mongo_connection.shinken.contacts.find_one(
            {"contact_name": contact_name}, {'_id': 0}
        )
        return contact.Contact(**c)

    def update(self, contact_name, contact):
        """Modify an existing contact."""
        contact_dict = contact.as_dict()
        if "contact_name" not in contact_dict.keys():
            contact_dict['contact_name'] = contact_name

        self.request.mongo_connection.shinken.contacts.update(
            {"contact_name": contact_name},
            {"$set": contact_dict},
            upsert=True
        )

    def delete(self, contact_name):
        """Delete existing contact."""
        self.request.mongo_connection.shinken.contacts.remove(
            {"contact_name": contact_name}
        )

    def create(self, contact):
        """Create a new contact."""
        self.request.mongo_connection.shinken.contacts.insert(
            contact.as_dict()
        )

    def get_all(self):
        """Return all contacts."""
        contacts = [c for c
                    in self.request.mongo_connection.
                    shinken.contacts.find(
                        {"register": {"$ne": "0"}},  # Don't return templates
                        {'_id': 0}
                    )]
        contacts = [contact.Contact(**c) for c in contacts]
        return contacts


# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class TimePeriod(types.Base):
    timeperiod_name = wsme.wsattr(wtypes.text, mandatory=True)
    exclude = wsme.wsattr(wtypes.text, mandatory=False)

    periods = wsme.wsattr(
        wtypes.DictType(wtypes.text, wtypes.text),
        mandatory=False
    )

    def __init__(self, **kwargs):
        super(TimePeriod, self).__init__(**kwargs)

        periods = [i for i in kwargs.items() if isinstance(i[0], str)
                   and i[0] not in ['timeperiod_name', 'exclude', 'periods']]
        if len(periods) > 0:
            self.periods = {}
            for item in periods:
                self.periods[item[0]] = item[1]

    def as_dict(self):
        timeperiod_dict = super(TimePeriod, self).as_dict()
        periods = timeperiod_dict.pop("periods", None)
        if periods:
            for item in periods.items():
                timeperiod_dict[item[0]] = item[1]
        return timeperiod_dict

    @classmethod
    def sample(cls):
        return cls(
            timeperiod_name='nonworkhours',
            periods={
                "sunday": "0:00-24:00",
                "february 10": "00:00-24:00"
            }
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see `<http://www.gnu.org/licenses/>`.

"""Script to push a Shinken pack to Surveil"""

import argparse
import fnmatch
import os
import sys

from pymongo import mongo_client
from alignak.objects import config


def main():
    # Parse the arguments
    parser = argparse.ArgumentParser(
        prog='surveil-pack-upload',
        add_help=False,
    )
    parser.add_argument('--mongo-url',
                        default='localhost',
                        help='Defaults to localhost', type=str)
    parser.add_argument('--mongo-port',
                        default=27017,
                        help='Defaults to 27017', type=int)
    parser.add_argument('pack', metavar='[Pack]', type=str, nargs=1,
                        help='Pack directory')

    (options, args) = parser.parse_known_args(sys.argv[1:])

    pack_dir = options.pack[0]
    pack_sub_dir = os.path.join(pack_dir, 'pack')
    pack_name = os.path.basename(os.path.normpath(pack_dir))

    # Find the .cfg files
    cfg_files = [
        os.path.join(dirpath, f)
        for dirpath, dirnames, files in os.walk(pack_sub_dir)
        for f in fnmatch.filter(files, '*.cfg')
    ]

    # Load the config
    conf = config.Config()
    loaded_conf = conf.read_config(cfg_files)
    raw_objects = conf.read_config_buf(loaded_conf)

    # Remove the empty items
    non_empty_config = {k: v for k, v in raw_objects.items() if v}

    for config_type in non_empty_config:
        for config_item in non_empty_config[config_type]:
            # Tag the config objects
            config_item['SURVEIL_PACK_NAME'] = pack_name

            # Replace lists with csv
            items_to_modify = (
                [i for i in config_item.items() if isinstance(i[1], list)]
            )
            for i in items_to_modify:
                config_item[i[0]] = ','.join(i[1])

    # Remove the existing pack from mongodb
    mongo = mongo_client.MongoClient(host=options.mongo_url,
                                     port=options.mongo_port)
    mongo_shinken = mongo.shinken
    for collection in (
            mongo_shinken.collection_names(include_system_collections=False)
    ):
        mongo_shinken[collection].remove({'SURVEIL_PACK_NAME': pack_name})

    # Add the replacement pack
    for config_type in non_empty_config:
            mongo_shinken[config_type + 's'].insert(
                non_empty_config[config_type]
            )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.controllers.v2 import actions as v2_actions
from surveil.api.controllers.v2 import admin as v2_admin
from surveil.api.controllers.v2 import auth as v2_auth
from surveil.api.controllers.v2 import bansho as v2_bansho
from surveil.api.controllers.v2 import config as v2_config
from surveil.api.controllers.v2 import hello as v2_hello
from surveil.api.controllers.v2 import logs as v2_logs
from surveil.api.controllers.v2 import status as v2_status


class V2Controller(object):
    """Version 2 API controller root."""
    actions = v2_actions.ActionsController()
    config = v2_config.ConfigController()
    hello = v2_hello.HelloController()
    status = v2_status.StatusController()
    surveil = v2_admin.AdminController()
    auth = v2_auth.AuthController()
    logs = v2_logs.LogsController()
    bansho = v2_bansho.BanshoController()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


def build_influxdb_query(metric_name,
                         time_delta,
                         host_name=None,
                         service_description=None
                         ):
    group_by = []
    query = ['SELECT * FROM metric_%s'
             % metric_name]
    begin = time_delta.begin
    end = time_delta.end
    query.append("WHERE time >= '%s' AND time <= '%s'" % (begin, end))

    if host_name is None:
        group_by.append('host_name')
    else:
        query.append("AND host_name ='%s'" % host_name)

    if service_description is None:
        group_by.append('service_description')
    else:
        query.append("AND service_description ='%s'" % service_description)

    if len(group_by) != 0:
        query.append('GROUP BY')
        query.append(', '.join(group_by))

    query.append('ORDER BY time DESC')
    return ' '.join(query)


# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from surveil.api.datamodel.status.metrics import time_delta
from surveil.api.handlers.status.metrics import influxdb_time_query
from surveil.tests import base


class InfluxdbTimeQueryTest(base.BaseTestCase):
    def test_build_query_basic(self):
        query_time = time_delta.TimeDelta(begin='2015-01-29T21:50:44Z',
                                          end='2015-01-29T22:50:44Z', )
        query_metric_name = 'pl'

        result = influxdb_time_query.build_influxdb_query(query_metric_name,
                                                          query_time
                                                          )
        expected = ("SELECT * "
                    "FROM metric_pl "
                    "WHERE time >= '2015-01-29T21:50:44Z' "
                    "AND time <= '2015-01-29T22:50:44Z' "
                    "GROUP BY host_name, "
                    "service_description ORDER BY time DESC")

        self.assert_count_equal_backport(result, expected)

    def test_build_query_host_name(self):
        query_time = time_delta.TimeDelta(begin='2015-01-29T21:50:44Z',
                                          end='2015-01-29T22:50:44Z', )
        query_metric_name = 'pl'
        query_host_name = 'localhost'

        result = influxdb_time_query.build_influxdb_query(query_metric_name,
                                                          query_time,
                                                          query_host_name
                                                          )
        expected = ("SELECT * "
                    "FROM metric_pl "
                    "WHERE time >= '2015-01-29T21:50:44Z' "
                    "AND time <= '2015-01-29T22:50:44Z' "
                    "AND host_name ='localhost' "
                    "GROUP BY service_description "
                    "ORDER BY time DESC")

        self.assert_count_equal_backport(result, expected)

    def test_build_query_complete(self):
        query_time = time_delta.TimeDelta(begin='2015-01-29T21:50:44Z',
                                          end='2015-01-29T22:50:44Z', )
        query_metric_name = 'pl'
        query_host_name = 'localhost'
        query_service_description = 'mySQL'

        result = influxdb_time_query.build_influxdb_query(
            query_metric_name,
            query_time,
            query_host_name,
            query_service_description
        )
        expected = ("SELECT * "
                    "FROM metric_pl "
                    "WHERE time >= '2015-01-29T21:50:44Z' "
                    "AND time <= '2015-01-29T22:50:44Z' "
                    "AND host_name ='localhost' "
                    "AND service_description ='mySQL' "
                    "ORDER BY time DESC")

        self.assert_count_equal_backport(result, expected)


# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class LiveMetric(types.Base):

    metric_name = wsme.wsattr(wtypes.text, mandatory=True)
    """Name of the metric"""

    max = wsme.wsattr(wtypes.text, mandatory=False)
    """Maximum value for the metric"""

    min = wsme.wsattr(wtypes.text, mandatory=False)
    """Minimal value for the metric"""

    critical = wsme.wsattr(wtypes.text, mandatory=False)
    """Critical value for the metric"""

    warning = wsme.wsattr(wtypes.text, mandatory=False)
    """Warning value for the metric"""

    value = wsme.wsattr(wtypes.text, mandatory=False)
    """Current value of the metric"""

    unit = wsme.wsattr(wtypes.text, mandatory=False)
    """Unit of the metric"""

    @classmethod
    def sample(cls):
        return cls(
            metric_name='pl',
            max='100',
            min='0',
            critical='100',
            warning='100',
            value='0',
            unit='s'
        )



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json


from surveil.tests.api import functionalTest


class TestNotificationWayController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestNotificationWayController, self).setUp()
        self.notificationway = [
            {
                'notificationway_name': 'email_in_day',
                'host_notification_period': '24x7',
                'service_notification_period': '24x7',
                'host_notification_options': 'd,u',
                'service_notification_options': 'w,c,r',
                'host_notification_commands': 'notify-service',
                'service_notification_commands': 'notify-host'
            },
            {
                'notificationway_name': 'email_all_time',
                'host_notification_period': '24x7',
                'service_notification_period': '24x7',
                'host_notification_options': 'd,r,f,u',
                'service_notification_options': 'w,f,c,r',
                'host_notification_commands': 'notify-service',
                'service_notification_commands': 'notify-host',
                'min_business_impact': 5
            }
        ]
        self.mongoconnection.shinken.notificationways.insert(
            copy.deepcopy(self.notificationway)
        )

    def test_get_all_notificationways(self):
        response = self.get('/v2/config/notificationways')

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            [
                {
                    'notificationway_name': 'email_in_day',
                    'host_notification_period': '24x7',
                    'service_notification_period': '24x7',
                    'host_notification_options': 'd,u',
                    'service_notification_options': 'w,c,r',
                    'host_notification_commands': 'notify-service',
                    'service_notification_commands': 'notify-host'
                },
                {
                    'notificationway_name': 'email_all_time',
                    'host_notification_period': '24x7',
                    'service_notification_period': '24x7',
                    'host_notification_options': 'd,r,f,u',
                    'service_notification_options': 'w,f,c,r',
                    'host_notification_commands': 'notify-service',
                    'service_notification_commands': 'notify-host',
                    'min_business_impact': 5
                }
            ]
        )
        self.assertEqual(response.status_int, 200)

    def test_get_one_notificationway(self):
        response = self.get('/v2/config/notificationways/email_all_time')

        self.assertEqual(
            json.loads(response.body.decode()),
            {
                'notificationway_name': 'email_all_time',
                'host_notification_period': '24x7',
                'service_notification_period': '24x7',
                'host_notification_options': 'd,r,f,u',
                'service_notification_options': 'w,f,c,r',
                'host_notification_commands': 'notify-service',
                'service_notification_commands': 'notify-host',
                'min_business_impact': 5
            }
        )

    def test_create_notificationway(self):
        notificationway = {
            'notificationway_name': 'test_create_notification',
            'host_notification_period': '24x7',
            'service_notification_period': '24x7',
            'host_notification_options': 'd,r,f,u',
            'service_notification_options': 'w,f,c,r',
            'host_notification_commands': 'notify-service',
            'service_notification_commands': 'notify-host',
            'min_business_impact': 5
        }

        self.post_json('/v2/config/notificationways', notificationway)

        self.assertIsNotNone(
            self.mongoconnection.
            shinken.notificationways.find_one(notificationway)
        )

    def test_delete_notificationway(self):
        notificationway = {
            'notificationway_name': 'email_all_time',
        }

        self.assertIsNotNone(
            self.mongoconnection.shinken.notificationways.find_one(
                notificationway
            )
        )

        self.delete('/v2/config/notificationways/email_all_time')

        self.assertIsNone(
            self.mongoconnection.shinken.notificationways.find_one(
                notificationway
            )
        )

    def test_put_notificationway(self):
        self.assertEqual(
            self.mongoconnection.shinken.notificationways.find_one(
                {'notificationway_name': 'email_all_time'}
            )['min_business_impact'],
            5
        )

        self.put_json(
            '/v2/config/notificationways/email_all_time',
            {
                'notificationway_name': 'email_all_time',
                'host_notification_period': '24x7',
                'service_notification_period': '24x7',
                'host_notification_options': 'd,r,f,u',
                'service_notification_options': 'w,f,c,r',
                'host_notification_commands': 'notify-service',
                'service_notification_commands': 'notify-host',
                'min_business_impact': 3
            }
        )

        self.assertEqual(
            self.mongoconnection.shinken.notificationways.find_one(
                {'notificationway_name': 'email_all_time'}
                )['min_business_impact'],
            3
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import hostgroup
from surveil.api.handlers import handler


class HostGroupHandler(handler.Handler):
    """Fulfills a request on the host group resource."""

    def get(self, group_name):
        """Return a host group."""

        g = self.request.mongo_connection.shinken.hostgroups.find_one(
            {"hostgroup_name": group_name}, {'_id': 0}
        )
        return hostgroup.HostGroup(**g)

    def update(self, group_name, group):
        """Modify an existing host group."""
        group_dict = group.as_dict()
        if "hostgroup_name" not in group_dict.keys():
            group_dict['hostgroup_name'] = group_name

        self.request.mongo_connection.shinken.hostgroups.update(
            {"hostgroup_name": group_name},
            {"$set": group_dict},
            upsert=True
        )

    def delete(self, group_name):
        """Delete existing host group."""
        self.request.mongo_connection.shinken.hostgroups.remove(
            {"hostgroup_name": group_name}
        )

    def create(self, group):
        """Create a new host group."""
        self.request.mongo_connection.shinken.hostgroups.insert(
            group.as_dict()
        )

    def get_all(self):
        """Return all host groups."""
        hostgroups = [g for g
                      in self.request.mongo_connection.
                      shinken.hostgroups.find(
                          {"register": {"$ne": "0"}},
                          {'_id': 0}
                      )]
        hostgroups = [hostgroup.HostGroup(**g) for g in hostgroups]
        return hostgroups



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from surveil.api.datamodel.status.metrics import live_metric
from surveil.api.handlers import handler
from surveil.api.handlers.status.metrics import influxdb_time_query


class MetricHandler(handler.Handler):
    """Fulfills a request on the metrics."""

    def get_metric(self, host_name, service_description=None):
        """Return all metrics name for a given host."""

        cli = self.request.influxdb_client

        if service_description is None:
            query = "SHOW measurements WHERE host_name='%s'" % host_name
        else:
            query = ("SHOW measurements WHERE host_name='%s' "
                     "AND service_description='%s'"
                     % (host_name, service_description))

        response = cli.query(query)

        metric_name_dicts = []

        for item in response[None]:
            metric_name_dict = self._metrics_name_from_influx_item(item)
            metric_name_dicts.append(metric_name_dict)

        metrics = []
        for metric_dict in metric_name_dicts:
            metric = live_metric.LiveMetric(**metric_dict)
            metrics.append(metric)

        return metrics

    def get(self, metric_name, host_name, service_description=None):
        """Return a metric."""

        cli = self.request.influxdb_client
        if service_description is None:
            query = ("SELECT * FROM metric_%s "
                     "WHERE host_name= '%s' "
                     "GROUP BY service_description "
                     "ORDER BY time DESC "
                     "LIMIT 1" % (metric_name, host_name))
        else:
            query = ("SELECT * FROM metric_%s "
                     "WHERE host_name= '%s' "
                     "AND service_description= '%s'"
                     "ORDER BY time DESC "
                     "LIMIT 1" % (metric_name, host_name, service_description))

        response = cli.query(query)
        metric = live_metric.LiveMetric(
            **self._metric_dict_from_influx_item(next(response.items()[0][1]),
                                                 metric_name)
        )

        return metric

    def get_all(self, metric_name, time_delta, host_name=None,
                service_description=None):
        """Return all metrics."""

        cli = self.request.influxdb_client
        query = influxdb_time_query.build_influxdb_query(
            metric_name,
            time_delta,
            host_name,
            service_description
        )
        response = cli.query(query)

        metric_dicts = []

        for item in response[None]:
            metric_dict = self._metric_dict_from_influx_item(item, metric_name)
            metric_dicts.append(metric_dict)

        metrics = []
        for metric_dict in metric_dicts:
            metric = live_metric.LiveMetric(**metric_dict)
            metrics.append(metric)

        return metrics

    def _metric_dict_from_influx_item(self, item, metric_name):

        metric_dict = {"metric_name": str(metric_name)}

        mappings = [
            ('min', str),
            ('max', str),
            ('critical', str),
            ('warning', str),
            ('value', str),
            ('unit', str),
        ]

        for field in mappings:
            value = item.get(field[0], None)
            if value is not None:
                metric_dict[field[0]] = field[1](value)

        return metric_dict

    def _metrics_name_from_influx_item(self, item):

        metric_name = {}
        mappings = [('metric_name', 'name', str), ]
        for field in mappings:
            value = item.get(field[1], None)
            if value is not None:
                metric_name[field[0]] = field[2](value)

        return metric_name


# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import influxdb
from pecan import hooks
import pymongo


class DBHook(hooks.PecanHook):

    def __init__(self, mongo_url, ws_arbiter_url, influxdb_url):
        self.mongo_url = mongo_url
        self.ws_arbiter_url = ws_arbiter_url
        self.influxdb_url = influxdb_url

    def before(self, state):
        self.mongoclient = pymongo.MongoClient(self.mongo_url)

        state.request.mongo_connection = self.mongoclient
        state.request.ws_arbiter_url = self.ws_arbiter_url
        state.request.influxdb_client = influxdb.InfluxDBClient.from_DSN(
            self.influxdb_url
        )

    def after(self, state):
        self.mongoclient.close()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys


class Plugin(object):

    def run(self):
        print("DISK OK - free space: / 3326 MB (56%); | /=2643MB;5948;5958;0;"
              "5968")
        sys.exit(0)


def main():
    Plugin().run()

if __name__ == "__main__":
    main()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import realm
from surveil.api.handlers import handler


class RealmHandler(handler.Handler):
    """Fulfills a request on the realm resource."""

    def get(self, realm_name):
        """Return a realm."""

        r = self.request.mongo_connection.shinken.realms.find_one(
            {"realm_name": realm_name}, {'_id': 0}
        )
        return realm.Realm(**r)

    def update(self, realm_name, realm):
        """Modify an existing realm."""
        realm_dict = realm.as_dict()
        if "realm_name" not in realm_dict.keys():
            realm_dict['realm_name'] = realm_name

        self.request.mongo_connection.shinken.realms.update(
            {"realm_name": realm_name},
            {"$set": realm_dict},
            upsert=True
        )

    def delete(self, realm_name):
        """Delete existing realm."""
        self.request.mongo_connection.shinken.realms.remove(
            {"realm_name": realm_name}
        )

    def create(self, realm):
        """Create a new realm."""
        self.request.mongo_connection.shinken.realms.insert(
            realm.as_dict()
        )

    def get_all(self):
        """Return all realms."""
        realms = [c for c
                  in self.request.mongo_connection.
                  shinken.realms.find(
                      {"register": {"$ne": "0"}},  # Don't return templates
                      {'_id': 0}
                  )]
        realms = [realm.Realm(**r) for r in realms]
        return realms



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest


from surveil.api.controllers.v2.logs import acknowledgements
from surveil.api.controllers.v2.logs import comments
from surveil.api.controllers.v2.logs import downtimes
from surveil.api.controllers.v2.logs import notifications
from surveil.common import util


class LogsController(rest.RestController):
    acknowledgements = acknowledgements.AcknowledgementsController()
    comments = comments.CommentsController()
    downtimes = downtimes.DowntimesController()
    notifications = notifications.NotificationsController()

    # @wsme_pecan.wsexpose([Host])
    @util.policy_enforce(['authenticated'])
    @pecan.expose()
    def get_all(self):
        """Returns all events from a specific host."""
        host_name = pecan.request.context.get("host_name")
        if host_name is not None:
            return "All events for %s" % host_name
        return "ALLL Events"

    # @pecan.expose()
    # def _lookup(self, host_name, *remainder):
    #    return EventController(host_name), remainder



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys
import time
import unittest


class BaseTestCase(unittest.TestCase):
    maxDiff = None

    def assert_count_equal_backport(self, item1, item2):
        if sys.version_info[0] >= 3:
            result = self.assertCountEqual(
                item1,
                item2
            )
        else:
            result = self.assertItemsEqual(
                sorted(item1),
                sorted(item2)
            )

        return result

    def try_for_x_seconds(self,
                          function,
                          time_to_wait=108,
                          message="Function did not succeed",
                          cooldown=10,
                          exception=Exception):
        """Returns True if the functions raises no exception."""

        now = time.time()
        while True:
            if time.time() < (now + time_to_wait):
                try:
                    function()
                    return True
                except exception:
                    pass
                time.sleep(cooldown)
            else:
                raise Exception(message)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest

from surveil.common import util


class NotificationsController(rest.RestController):

    # curl -X GET  http://127.0.0.1:8080/v2/titilambert/myproject/builds/
    # @wsme_pecan.wsexpose([Host])
    @util.policy_enforce(['authenticated'])
    @pecan.expose()
    def get_all(self):
        """Returns all notifications from a specific host."""
        return "ALLL notifs"


# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Starter script for the Surveil API service."""

from __future__ import print_function
from __future__ import unicode_literals
import os
import subprocess
import sys
import threading
import time
from wsgiref import simple_server

from oslo_config import cfg

import surveil.api.app as app

CONF = cfg.CONF

OPTS = [
    cfg.StrOpt(
        'api_paste_config',
        default="api_paste.ini",
        help="Configuration file for WSGI definition of API."
    ),
]

CONF.register_opts(OPTS)


class ServerManager:

    def __init__(self):
        self.config = {}
        self.config_file = ""
        self.server_process = None
        self.should_run = True

    def run(self, pecan_config, config_file):
        self.config = pecan_config
        self.config_file = config_file

        if '--reload' in sys.argv:
            self.watch_and_spawn()
        else:
            self.start_server()

    def create_subprocess(self):
        self.server_process = subprocess.Popen(['surveil-api'])

    def start_server(self):
        pecan_app = app.load_app()
        host, port = self.config.server.host, self.config.server.port
        srv = simple_server.make_server(host, port, pecan_app)
        srv.serve_forever()

    def watch_and_spawn(self):
        import watchdog.events as events
        import watchdog.observers as observers

        print('Monitoring for changes...',
              file=sys.stderr)

        self.create_subprocess()
        parent = self

        class AggressiveEventHandler(events.FileSystemEventHandler):

            def __init__(self):
                self.wait = False

            def should_reload(self, event):
                for t in (
                    events.FileSystemMovedEvent,
                    events.FileModifiedEvent,
                    events.DirModifiedEvent
                ):
                    if isinstance(event, t):
                        return True
                return False

            def ignore_events_one_sec(self):
                if not self.wait:
                    self.wait = True
                    t = threading.Thread(target=self.wait_one_sec)
                    t.start()

            def wait_one_sec(self):
                time.sleep(1)
                self.wait = False

            def on_modified(self, event):
                if self.should_reload(event) and not self.wait:
                    print("Some source files have been modified",
                          file=sys.stderr)
                    print("Restarting server...",
                          file=sys.stderr)
                    parent.server_process.kill()
                    self.ignore_events_one_sec()
                    parent.create_subprocess()

        path = self.path_to_monitor()

        event_handler = AggressiveEventHandler()

        observer = observers.Observer()
        observer.schedule(
            event_handler,
            path=path,
            recursive=True
        )
        observer.start()

        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            pass

    def path_to_monitor(self):
        module = __import__('surveil')
        return os.path.dirname(module.__file__)


def main():
    srv = ServerManager()
    srv.run(app.get_pecan_config(), app.get_config_filename())



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json

import requests_mock
from six.moves import urllib_parse

from surveil.api.controllers.v1.datamodel import host
from surveil.tests.api import functionalTest


class TestHostController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestHostController, self).setUp()
        self.hosts = [
            {
                "host_name": "bogus-router", "address": "192.168.1.254",
                "max_check_attempts": 5, "check_period": "24x7",
                "contacts": "admin,carl", "contact_groups": "router-admins",
                "notification_interval": 30, "notification_period": "24x7"
            },
            {
                "host_name": "bogus-router2", "address": "192.168.1.254",
                "max_check_attempts": 5, "check_period": "24x7",
                "contacts": "admin,carl", "contact_groups": "router-admins",
                "notification_interval": 30, "notification_period": "24x7"
            },
            {
                "host_name": "bogus-router333", "address": "192.168.1.254",
                "max_check_attempts": 5, "check_period": "24x7",
                "contacts": "admin,carl", "contact_groups": "router-admins",
                "notification_interval": 30, "notification_period": "24x7"
            },
        ]
        self.mongoconnection.shinken.hosts.insert(
            copy.deepcopy(self.hosts)
        )

        self.services = [
            {
                "host_name": "bogus-router",
                "service_description": "service-example",
                "check_command": "check-disk!/dev/sdb1",
                "max_check_attempts": 5,
                "check_interval": 5,
                "retry_interval": 3,
                "check_period": "24x7",
                "notification_interval": 30,
                "notification_period": "24x7",
                "contacts": "surveil-ptl,surveil-bob",
                "contact_groups": "linux-admins"
            }
        ]
        self.mongoconnection.shinken.services.insert(
            copy.deepcopy(self.services)
        )

    def test_get_all_hosts(self):
        response = self.app.get('/v1/hosts')

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.hosts
        )
        self.assertEqual(response.status_int, 200)

    def test_get_all_hosts_no_templates(self):
        self.mongoconnection.shinken.hosts.insert(
            copy.deepcopy(
                {"host_name": "bogus-router", "address": "192.168.1.254",
                 "max_check_attempts": 5, "check_period": "24x7",
                 "contacts": "admin,carl", "contact_groups": "router-admins",
                 "notification_interval": 30, "notification_period": "24x7",
                 "register": "0"}
            )
        )
        response = self.app.get('/v1/hosts')

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.hosts
        )
        self.assertEqual(response.status_int, 200)

    def test_get_specific_host(self):
        response = self.app.get('/v1/hosts/bogus-router333')

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.hosts[2]
        )
        self.assertEqual(response.status_int, 200)

    def test_update_host(self):
        put_host = {
            u"host_name": u"bogus-router333",
            u"address": u"newputaddress",
            u"max_check_attempts": 222225,
            u"check_period": u"newtimeperiod",
            u"contacts": u"aaa,bbb",
            u"contact_groups": u"newgroup",
            u"notification_interval": 333,
            u"notification_period": u"newnotificationperiod"
        }
        response = self.app.put_json(
            "/v1/hosts/bogus-router333", params=put_host
        )

        mongo_host = host.Host(
            **self.mongoconnection.shinken.hosts.find_one(
                {'host_name': 'bogus-router333'}, {'_id': 0}
            )
        )

        self.assertEqual(put_host, mongo_host.as_dict())
        self.assertEqual(response.status_int, 204)

    def test_delete_host(self):
        response = self.app.delete('/v1/hosts/bogus-router')

        mongo_hosts = [host.Host(**h) for h
                       in self.mongoconnection.shinken.hosts.find()]

        self.assertEqual(2, len(mongo_hosts))
        self.assertEqual(response.status_int, 204)

    def test_add_host(self):
        new_host = {
            "host_name": "testpost",
            "address": "192.168.1.254",
            "max_check_attempts": 5,
            "check_period": "24x7",
            "contacts": "admin,carl",
            "contact_groups": "router-admins",
            "notification_interval": 3,
            "notification_period": "24x7"
        }
        response = self.app.post_json("/v1/hosts", params=new_host)

        hosts = [host.Host(**h).as_dict() for h
                 in self.mongoconnection.shinken.hosts.find(None, {'_id': 0})]

        self.assertTrue(new_host in hosts)
        self.assertEqual(response.status_int, 201)

    def test_get_associated_services(self):
        response = self.app.get('/v1/hosts/bogus-router/services')

        self.assertEqual(
            self.services,
            json.loads(response.body.decode())
        )

    def test_get_specific_service(self):
        response = self.app.get(
            '/v1/hosts/bogus-router/services/service-example'
        )

        self.assertEqual(
            self.services[0],
            json.loads(response.body.decode())
        )

    def test_submit_service_result(self):
        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.POST,
                           self.ws_arbiter_url + "/push_check_result")

            check_result = {
                "return_code": "0",
                "output": "TEST OUTPUT",
                "time_stamp": "1409149234"
            }

            response = self.app.post_json(
                "/v1/hosts/bogus-router/services/service-example/results",
                params=check_result
            )

            self.assertEqual(response.status_int, 204)
            self.assertEqual(
                urllib_parse.parse_qs(m.last_request.body),
                {
                    u'output': [u'TEST OUTPUT'],
                    u'return_code': [u'0'],
                    u'service_description': [u'service-example'],
                    u'host_name': [u'bogus-router'],
                    u'time_stamp': [u'1409149234']
                }
            )

    def test_submit_host_result(self):
        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.POST,
                           self.ws_arbiter_url + "/push_check_result")

            check_result = {
                "return_code": "0",
                "output": "TEST OUTPUT",
                "time_stamp": "1409149234"
            }

            response = self.app.post_json("/v1/hosts/bogus-router/results",
                                          params=check_result)

            self.assertEqual(response.status_int, 204)
            self.assertEqual(
                urllib_parse.parse_qs(m.last_request.body),
                {
                    u'output': [u'TEST OUTPUT'],
                    u'return_code': [u'0'],
                    u'host_name': [u'bogus-router'],
                    u'time_stamp': [u'1409149234']
                }
            )



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from surveil.tests.api import functionalTest


class TestAuthController(functionalTest.FunctionalTest):

    def test_auth_login(self):
        auth = {
            "auth": {
                "tenantName": "demo",
                "passwordCredentials": {
                    "username": "demo",
                    "password": "secretsecret"
                }
            }
        }

        response = self.post_json('/v2/auth/tokens', params=auth)

        expected = {
            "access": {
                "token": {
                    "issued_at": "2014-01-30T15:30:58.819584",
                    "expires": "2014-01-31T15:30:58Z",
                    "id": "aaaaa-bbbbb-ccccc-dddd",
                    "tenant": {
                        "enabled": True,
                        "description": "Hey!",
                        "name": "demo",
                        "id": "fc394f2ab2df4114bde39905f800dc57"
                    }
                }
            }
        }

        self.assertEqual(json.loads(response.body.decode()), expected)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from surveil.api.controllers.v1.datamodel import host
from surveil.tests import base


class TestHostDataModel(base.BaseTestCase):

    def test_host_as_and_from_dict(self):
        h = host.Host(
            host_name="bogus-router",
            address="192.168.1.254",
            max_check_attempts=5,
            check_period="24x7",
            contacts="admin,carl",
            contact_groups="router-admins",
            notification_interval=30,
            notification_period="24x7",
            custom_fields={"_OS_AUTH_URL": "http://localhost:8080/v2"}
        )

        h_dict = {'check_period': u'24x7', 'notification_interval': 30,
                  'contacts': u'admin,carl', 'notification_period': u'24x7',
                  'contact_groups': u'router-admins',
                  'host_name': u'bogus-router', 'address': u'192.168.1.254',
                  'max_check_attempts': 5,
                  '_OS_AUTH_URL': 'http://localhost:8080/v2'}

        self.assertEqual(h.as_dict(), h_dict)
        self.assertEqual(h_dict, host.Host(**h_dict).as_dict())



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import command
from surveil.api.handlers import handler


class CommandHandler(handler.Handler):
    """Fulfills a request on the service resource."""

    def get(self, command_name):
        """Return a command."""
        c = self.request.mongo_connection.shinken.commands.find_one(
            {"command_name": command_name}
        )
        return command.Command(**c)

    def update(self, command_name, command):
        """Modify existing command."""
        command_dict = command.as_dict()
        if "command_name" not in command_dict.keys():
            command_dict['command_name'] = command_name

        self.request.mongo_connection.shinken.commands.update(
            {"command_name": command_name},
            {"$set": command_dict},
            upsert=True
        )

    def delete(self, command_name):
        """Delete an existing command."""
        self.request.mongo_connection.shinken.commands.remove(
            {"command_name": command_name}
        )

    def create(self, data):
        """Create a new command."""
        self.request.mongo_connection.shinken.commands.insert(
            data.as_dict()
        )

    def get_all(self):
        """Return all commands."""
        commands = [c for c
                    in self.request.mongo_connection.shinken.commands.find()]

        return [command.Command(**c) for c in commands]


# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import requests_mock

from surveil.tests.api import functionalTest


class TestDowntimeController(functionalTest.FunctionalTest):

    def test_downtime_post(self):
        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.POST,
                           self.ws_arbiter_url + "/downtime")

            dt = {
                "host_name": "localhost",
                "duration": 86400
            }

            response = self.post_json("/v2/actions/downtime/", params=dt)

            self.assertEqual(response.status_int, 200)

            self.assert_count_equal_backport(m.last_request.body.split('&'),
                                             ['host_name=localhost',
                                              'action=add',
                                              'duration=86400'])
            self.assertEqual(m.last_request.path,
                             '/downtime')

    def test_downtime_delete(self):
        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.POST,
                           self.ws_arbiter_url + "/downtime")

            dt = {
                "host_name": "localhost",
                "duration": 86400
            }

            response = self.delete_json("/v2/actions/downtime/", params=dt)

            self.assertEqual(response.status_int, 200)

            self.assert_count_equal_backport(m.last_request.body.split('&'),
                                             ['host_name=localhost',
                                              'action=delete',
                                              'duration=86400'])
            self.assertEqual(m.last_request.path,
                             '/downtime')



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import host
from surveil.api.handlers import handler


class HostHandler(handler.Handler):
    """Fulfills a request on the host resource."""

    def get(self, host_name):
        """Return a host."""

        h = self.request.mongo_connection.shinken.hosts.find_one(
            {"host_name": host_name}, {'_id': 0}
        )
        return host.Host(**h)

    def update(self, host_name, host):
        """Modify existing host."""
        host_dict = host.as_dict()
        if "host_name" not in host_dict.keys():
            host_dict['host_name'] = host_name

        self.request.mongo_connection.shinken.hosts.update(
            {"host_name": host_name},
            {"$set": host_dict},
            upsert=True
        )

    def delete(self, host_name):
        """Delete existing host."""
        self.request.mongo_connection.shinken.hosts.remove(
            {"host_name": host_name}
        )

    def create(self, host):
        """Create a new host."""
        self.request.mongo_connection.shinken.hosts.insert(
            host.as_dict()
        )

    def get_all(self):
        """Return all hosts."""
        hosts = [h for h
                 in self.request.mongo_connection.
                 shinken.hosts.find(
                     {"register": {"$ne": "0"}},  # Don't return templates
                     {'_id': 0}
                 )]
        hosts = [host.Host(**h) for h in hosts]
        return hosts


# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import service
from surveil.api.handlers import handler


class ServiceHandler(handler.Handler):
    """Fulfills a request on the service resource."""

    def get(self, host_name, service_description):
        """Return a service."""
        mongo_s = self.request.mongo_connection.shinken.services.find_one(
            {"host_name": host_name,
             "service_description": service_description}
        )
        return service.Service(**mongo_s)

    def update(self, id, data):
        """Modify existing host."""
        host_dict = data.as_dict()
        if "host_name" not in host_dict.keys():
            host_dict['host_name'] = id

        self.request.mongo_connection.shinken.hosts.update(
            {"host_name": id},
            {"$set": host_dict},
            upsert=True
        )

    def delete(self, host_name, service_description):
        """Delete existing service."""
        self.request.mongo_connection.shinken.services.remove(
            {"host_name": host_name,
             "service_description": service_description}
        )

    def create(self, data):
        """Create a new service."""
        self.request.mongo_connection.shinken.services.insert(
            data.as_dict()
        )

    def get_all(self, host_name=None):
        """Return all services."""
        filters = {"register": {"$ne": "0"}}

        if host_name is not None:
            filters['host_name'] = host_name

        services = [
            s for s
            in self.request.mongo_connection.
            # Don't return templates
            shinken.services.find(filters)
        ]
        return [service.Service(**s) for s in services]



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import functools

import pecan
from webob import exc

from surveil.api import rbac


# TODO(aviau && Freddrickk): Properly document this decorator
def policy_enforce(actions):
    def policy_enforce_inner(handler):
        @functools.wraps(handler)
        def handle_stack_method(*args, **kwargs):
            request = pecan.request
            for action in actions:
                allowed = rbac.enforce(action, request)

                if not allowed:
                    raise exc.HTTPForbidden()

            return handler(*args, **kwargs)
        return handle_stack_method
    return policy_enforce_inner



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import pecan
from pecan import rest
import wsme.types as wtypes
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.config import timeperiod
from surveil.api.handlers.config import timeperiod_handler
from surveil.common import util


class TimePeriodsController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([timeperiod.TimePeriod])
    def get_all(self):
        """Returns all time periods."""
        handler = timeperiod_handler.TimePeriodHandler(pecan.request)
        time_periods = handler.get_all()
        return time_periods

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(timeperiod.TimePeriod, wtypes.text)
    def get_one(self, timeperiod_name):
        """Returns a specific time period."""
        handler = timeperiod_handler.TimePeriodHandler(pecan.request)
        timeperiod = handler.get(timeperiod_name)
        return timeperiod

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=timeperiod.TimePeriod, status_code=201)
    def post(self, data):
        """Create a new time period.

        :param data: a time period within the request body.
        """
        handler = timeperiod_handler.TimePeriodHandler(pecan.request)
        handler.create(data)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(timeperiod.TimePeriod, wtypes.text, status_code=204)
    def delete(self, timeperiod_name):
        """Returns a specific time period."""
        handler = timeperiod_handler.TimePeriodHandler(pecan.request)
        handler.delete(timeperiod_name)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(timeperiod.TimePeriod,
                         wtypes.text,
                         body=timeperiod.TimePeriod,
                         status_code=204)
    def put(self, timeperiod_name, timeperiod):
        """Update a specific time period."""
        handler = timeperiod_handler.TimePeriodHandler(pecan.request)
        handler.update(timeperiod_name, timeperiod)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


# import shinken.objects
# Command = type(
#     'Command',
#     (wtypes.Base,),
#     {key: wtypes.text for key in shinken.objects.Command.properties.keys()}
# )


class Command(types.Base):
    command_name = wsme.wsattr(wtypes.text, mandatory=True)
    """The name of the command"""

    command_line = wsme.wsattr(wtypes.text, mandatory=True)
    """This directive is used to define what is actually executed by Shinken"""

    @classmethod
    def sample(cls):
        return cls(
            command_name="check_http",
            command_line="/bin/check_http"
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import requests
import wsmeext.pecan as wsme_pecan

from surveil.api.controllers.v1.datamodel import checkresult
from surveil.api.controllers.v1.datamodel import host
from surveil.api.controllers.v1.datamodel import service


class ServiceCheckResultsSubController(rest.RestController):

    @wsme_pecan.wsexpose(body=checkresult.CheckResult, status_code=204)
    def post(self, data):
        """Submit a new check result.

        :param data: a check result within the request body.
        """
        result = data.as_dict()
        result['host_name'] = pecan.request.context['host_name']

        result['service_description'] = pecan.request.context[
            'service_description'
        ]

        requests.post(
            pecan.request.ws_arbiter_url + "/push_check_result",
            data=result
        )


class HostServiceSubController(rest.RestController):
    results = ServiceCheckResultsSubController()

    def __init__(self, service_description):
        pecan.request.context['service_description'] = service_description
        self._id = service_description

    @wsme_pecan.wsexpose(service.Service)
    def get(self):
        """Returns a specific service."""
        mongo_s = pecan.request.mongo_connection.shinken.services.find_one(
            {
                "host_name": pecan.request.context['host_name'],
                "service_description": pecan.request.context[
                    'service_description'
                ]
            }
        )

        return service.Service(**mongo_s)


class HostServicesSubController(rest.RestController):

    @wsme_pecan.wsexpose([service.Service])
    def get_all(self):
        """Returns all services assocaited with this host."""
        mongo_s = [
            s for s
            in pecan.request.mongo_connection.shinken.services.find(
                {"host_name": pecan.request.context['host_name']}
            )
        ]

        services = [service.Service(**s) for s in mongo_s]

        return services

    @pecan.expose()
    def _lookup(self, service_description, *remainder):
        return HostServiceSubController(service_description), remainder


class HostCheckResultsSubController(rest.RestController):

    @wsme_pecan.wsexpose(body=checkresult.CheckResult, status_code=204)
    def post(self, data):
        """Submit a new check result.

        :param data: a check result within the request body.
        """
        result = data.as_dict()
        result['host_name'] = pecan.request.context['host_name']

        requests.post(
            pecan.request.ws_arbiter_url + "/push_check_result",
            data=result
        )


class HostSubController(rest.RestController):
    services = HostServicesSubController()
    results = HostCheckResultsSubController()


class HostController(rest.RestController):

    def __init__(self, host_name):
        pecan.request.context['host_name'] = host_name
        self._id = host_name

    @wsme_pecan.wsexpose(host.Host)
    def get(self):
        """Returns a specific host."""
        h = pecan.request.mongo_connection.shinken.hosts.find_one(
            {"host_name": self._id}, {'_id': 0}
        )
        return host.Host(**h)

    @wsme_pecan.wsexpose(None, body=host.Host, status_code=204)
    def put(self, data):
        """Modify this host.

        :param data: a host within the request body.
        """

        host_dict = data.as_dict()
        if "host_name" not in host_dict.keys():
            host_dict['host_name'] = self._id

        pecan.request.mongo_connection.shinken.hosts.update(
            {"host_name": self._id},
            host_dict
        )

    @wsme_pecan.wsexpose(None, status_code=204)
    def delete(self):
        """Delete this host."""
        pecan.request.mongo_connection.shinken.hosts.remove(
            {"host_name": self._id}
        )

    @pecan.expose()
    def _lookup(self, *remainder):
        return HostSubController(), remainder


class HostsController(rest.RestController):

    @pecan.expose()
    def _lookup(self, host_name, *remainder):
        return HostController(host_name), remainder

    @wsme_pecan.wsexpose([host.Host])
    def get_all(self):
        """Returns all hosts."""
        hosts = [h for h
                 in pecan.request.mongo_connection.
                 shinken.hosts.find(
                     {"register": {"$ne": "0"}},  # Don't return templates
                     {'_id': 0}
                 )]

        return [host.Host(**h) for h in hosts]

    @wsme_pecan.wsexpose(host.Host, body=host.Host, status_code=201)
    def post(self, data):
        """Create a new host.

        :param data: a host within the request body.
        """
        pecan.request.mongo_connection.shinken.hosts.insert(
            data.as_dict()
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json


def build_influxdb_query(live_query,
                         measurement,
                         group_by=[],
                         order_by=[],
                         limit=None):

    query = ['SELECT * FROM', measurement]

    if live_query:
        filters = json.loads(live_query.filters)
        if filters:
            query.append(_build_where_clause(filters))

    if group_by:
        query.append('GROUP BY')
        query.append(', '.join(group_by))

    if order_by:
        query.append('ORDER BY')
        query.append(', '.join(order_by))

    if limit is not None:
        query.append('LIMIT %d' % limit)

    return ' '.join(query)


def _build_where_clause(filters):
    filters_conversion = {
        'is': '=',
        'isnot': '!='
    }
    clause = []
    first = True

    for filter_name, filter_data in sorted(filters.items()):
        for field, values in sorted(filter_data.items()):
            for value in values:
                if first:
                    clause.append('WHERE')
                else:
                    clause.append('AND')

                if type(value) == int:
                    clause.append("%s%s%d" % (field,
                                              filters_conversion[filter_name],
                                              value))
                else:
                    clause.append("%s%s'%s'" %
                                  (field,
                                   filters_conversion[filter_name],
                                   value))
                first = False

    return ' '.join(clause)



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import notificationway
from surveil.api.handlers import handler


class NotificationWayHandler(handler.Handler):
    def get(self, notificationway_name):
        """Return a notification way."""

        g = self.request.mongo_connection.shinken.notificationways.find_one(
            {"notificationway_name": notificationway_name}, {'_id': 0}
        )
        return notificationway.NotificationWay(**g)

    def update(self, notificationway_name, notificationway):
        """Modify an existing notification way."""
        notificationway_dict = notificationway.as_dict()
        if "notificationway_name" not in notificationway_dict.keys():
            notificationway_dict['notificationway_name'] = notificationway_name

        self.request.mongo_connection.shinken.notificationways.update(
            {"notificationway_name": notificationway_name},
            {"$set": notificationway_dict},
            upsert=True
        )

    def delete(self, notificationway_name):
        """Delete existing notification way."""
        self.request.mongo_connection.shinken.notificationways.remove(
            {"notificationway_name": notificationway_name}
        )

    def create(self, notificationway):
        """Create a new notification way."""
        self.request.mongo_connection.shinken.notificationways.insert(
            notificationway.as_dict()
        )

    def get_all(self):
        """Return all notification way."""
        notificationways = [
            g for g in self.request.mongo_connection
            .shinken.notificationways.find(
                {"register": {"$ne": "0"}},
                {'_id': 0}
            )]
        notificationways = [
            notificationway.NotificationWay(**g) for g in notificationways
        ]

        return notificationways



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


def build_mongodb_query(lq_filters, lq_fields):
    #  Build the query
    query = {}
    for filter_name, filter_data in lq_filters.items():
        for field, values in filter_data.items():
            query[field] = {
                _get_mongo_filter(filter_name): values
            }

    #  Build the required fields
    fields = {}
    for field in lq_fields:
        fields[field] = 1

    return query, fields


def _get_mongo_filter(livequery_filter):
    filters = {
        "is": "$in",
        "isnot": "$nin"
    }
    return filters[livequery_filter]



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class LiveHost(types.Base):
    host_name = wsme.wsattr(wtypes.text, mandatory=False)
    """The name of the host"""

    address = wsme.wsattr(wtypes.text, mandatory=False)
    """The address of the host"""

    childs = wsme.wsattr([wtypes.text], mandatory=False)
    """The childs of the host"""

    parents = wsme.wsattr([wtypes.text], mandatory=False)
    """The parents of the host"""

    description = wsme.wsattr(wtypes.text, mandatory=False)
    """The description of the host"""

    state = wsme.wsattr(wtypes.text, mandatory=False)
    """The current state of the host"""

    acknowledged = wsme.wsattr(bool, mandatory=False)
    """Wether or not the problem, if any, has been acknowledged"""

    last_check = wsme.wsattr(int, mandatory=False)
    """The last time the host was checked"""

    last_state_change = wsme.wsattr(int, mandatory=False)
    """The last time the state has changed"""

    plugin_output = wsme.wsattr(wtypes.text, mandatory=False)
    """Plugin output of the last check"""

    long_output = wsme.wsattr(wtypes.text, mandatory=False)
    """Plugin long output of the last check"""

    services = wsme.wsattr([wtypes.text], mandatory=False)
    """The services of the host"""

    @classmethod
    def sample(cls):
        return cls(
            host_name='CoolHost',
            address="127.0.0.1",
            childs=['surveil.com'],
            parents=['parent.com'],
            description='Very Nice Host',
            state='OK',
            acknowledged=True,
            last_check=1429220785,
            last_state_change=1429220785,
            plugin_output='PING OK - Packet loss = 0%, RTA = 0.02 ms',
            long_output='The ping was great\nI love epic ping-pong games',
            services=['load', 'cpu', 'disk_usage']
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json


from surveil.api.datamodel.config import contact
from surveil.tests.api import functionalTest


class TestContactsController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestContactsController, self).setUp()
        self.contacts = [
            {
                'contact_name': 'bobby',
                'email': 'bob@bob.com'
            },
            {
                'contact_name': 'marie',
                'email': 'marie@marie.com'
            },
        ]
        self.mongoconnection.shinken.contacts.insert(
            copy.deepcopy(self.contacts)
        )

    def test_get_all_contacts(self):
        response = self.get('/v2/config/contacts')

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.contacts
        )
        self.assertEqual(response.status_int, 200)

    def test_get_one_contact(self):
        response = self.get('/v2/config/contacts/bobby')

        self.assertEqual(
            json.loads(response.body.decode()),
            self.contacts[0]
        )

    def test_create_contact(self):
        c = contact.Contact(
            contact_name='John'
        )

        self.post_json('/v2/config/contacts', c.as_dict())

        self.assertIsNotNone(
            self.mongoconnection.shinken.contacts.find_one(c.as_dict())
        )

    def test_delete_contact(self):
        self.assertIsNotNone(
            self.mongoconnection.shinken.contacts.find_one(self.contacts[0])
        )

        self.delete('/v2/config/contacts/bobby')

        self.assertIsNone(
            self.mongoconnection.shinken.contacts.find_one(self.contacts[0])
        )

    def test_put_contact(self):
        self.assertEqual(
            self.mongoconnection.shinken.contacts.find_one(
                {'contact_name': 'bobby'}
            )['email'],
            'bob@bob.com'
        )

        self.put_json(
            '/v2/config/contacts/bobby',
            {"contact_name": "bobby", "email": "updated@bob.com"}
        )

        self.assertEqual(
            self.mongoconnection.shinken.contacts.find_one(
                {'contact_name': 'bobby'}
            )['email'],
            'updated@bob.com'
        )



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import pecan
from pecan import rest
import wsme.types as wtypes
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.config import checkmodulation
from surveil.api.handlers.config import checkmodulation_handler
from surveil.common import util


class CheckModulationsController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([checkmodulation.CheckModulation])
    def get_all(self):
        """Returns all  check modulations."""
        handler = checkmodulation_handler.CheckModulationHandler(pecan.request)
        checkmodulations = handler.get_all()
        return checkmodulations

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(checkmodulation.CheckModulation, wtypes.text)
    def get_one(self, checkmodulation_name):
        """Returns a specific check modulation."""
        handler = checkmodulation_handler.CheckModulationHandler(pecan.request)
        checkmodulation = handler.get(checkmodulation_name)
        return checkmodulation

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=checkmodulation.CheckModulation, status_code=201)
    def post(self, data):
        """Create a new check modulation.

        :param data: a check modulation within the request body.
        """
        handler = checkmodulation_handler.CheckModulationHandler(pecan.request)
        handler.create(data)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(checkmodulation.CheckModulation,
                         wtypes.text, status_code=204)
    def delete(self, checkmodulation_name):
        """Returns a specific check modulation."""
        handler = checkmodulation_handler.CheckModulationHandler(pecan.request)
        handler.delete(checkmodulation_name)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(checkmodulation.CheckModulation,
                         wtypes.text,
                         body=checkmodulation.CheckModulation,
                         status_code=204)
    def put(self, checkmodulation_name, checkmodulation):
        """Update a specific check modulation."""
        handler = checkmodulation_handler.CheckModulationHandler(pecan.request)
        handler.update(checkmodulation_name, checkmodulation)


# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json


from surveil.tests.api import functionalTest


class TestCheckModulationsController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestCheckModulationsController, self).setUp()
        self.checkmodulations = [
            {
                'checkmodulation_name': 'ping_night',
                'check_command': 'check_ping_night',
                'check_period': 'night'
            },
            {
                'checkmodulation_name': 'ping_day',
                'check_command': 'check_ping_day',
                'check_period': 'day',
            },
        ]
        self.mongoconnection.shinken.checkmodulations.insert(
            copy.deepcopy(self.checkmodulations)
        )

    def test_get_all_checkmodulations(self):
        response = self.get('/v2/config/checkmodulations')

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            [
                {'checkmodulation_name': 'ping_day',
                 'check_command': 'check_ping_day',
                 'check_period': 'day'},
                {'checkmodulation_name': 'ping_night',
                 'check_command': 'check_ping_night',
                 'check_period': 'night'}
            ]
        )
        self.assertEqual(response.status_int, 200)

    def test_get_one_checkmodulation(self):
        response = self.get('/v2/config/checkmodulations/ping_night')

        self.assertEqual(
            json.loads(response.body.decode()),
            {'checkmodulation_name': 'ping_night',
             'check_command': 'check_ping_night',
             'check_period': 'night'}
        )

    def test_create_checkmodulation(self):
        t = {"checkmodulation_name": "ping_evening",
             "check_command": "check_ping_evening",
             "check_period": "evening"
             }

        self.post_json('/v2/config/checkmodulations', t)
        self.assertIsNotNone(
            self.mongoconnection.shinken.checkmodulations.find_one(
                {"checkmodulation_name": 'ping_evening',
                 "check_command": "check_ping_evening",
                 "check_period": "evening"})
        )

    def test_delete_checkmodulation(self):
        self.assertIsNotNone(
            self.mongoconnection.shinken.checkmodulations.find_one(
                {"checkmodulation_name": 'ping_night'}
            )
        )

        self.delete('/v2/config/checkmodulations/ping_night')

        self.assertIsNone(
            self.mongoconnection.shinken.checkmodulations.find_one(
                {"checkmodulation_name": 'ping_night'}
            )
        )

    def test_put_checkmodulation(self):
        self.assertEqual(
            self.mongoconnection.shinken.checkmodulations.find_one(
                {'checkmodulation_name': 'ping_night'}
            )['check_command'],
            'check_ping_night'
        )

        self.put_json(
            '/v2/config/checkmodulations/ping_night',
            {"checkmodulation_name": "ping_night",
             "check_command": "updated",
             "check_period": 'night'}
        )

        self.assertEqual(
            self.mongoconnection.shinken.checkmodulations.find_one(
                {'checkmodulation_name': 'ping_night'}
            )['check_command'],
            'updated'
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from surveil.api.controllers.v1.datamodel import service
from surveil.common import util


class ServicesController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([service.Service])
    def get_all(self):
        """Returns all services."""
        services = [
            s for s
            in pecan.request.mongo_connection.
            # Don't return templates
            shinken.services.find({"register": {"$ne": "0"}})
        ]

        return [service.Service(**s) for s in services]

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(service.Service,
                         body=service.Service,
                         status_code=201)
    def post(self, data):
        """Create a new service.

        :param data: a service within the request body.
        """
        pecan.request.mongo_connection.shinken.services.insert(
            data.as_dict()
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import requests

from surveil.tests.integration import integration_test


class TestMergedIngegrationSurveil(
    integration_test.MergedIntegrationTest
):

    def test_hello(self):
        self.assertEqual(
            requests.get("http://localhost:8999/v2/hello").text,
            'Hello World!'
        )


class TestSeparatedIntegrationSurveil(
    integration_test.SeparatedIntegrationTests
):

    def test_create_host(self):
        """Creates a host and asserts that is is monitored by Alignak."""
        config_hosts = (TestSeparatedIntegrationSurveil.
                        client.status.hosts.list())

        self.assertFalse(
            any(host['host_name'] == 'integrationhosttest'
                for host in config_hosts)
        )

        TestSeparatedIntegrationSurveil.client.config.hosts.create(
            host_name='integrationhosttest',
            address='127.0.0.1',
        )

        TestSeparatedIntegrationSurveil.client.config.reload_config()

        def function():
            status_hosts = (TestSeparatedIntegrationSurveil.
                            client.status.hosts.list())
            self.assertTrue(
                any(host['host_name'].decode() == 'integrationhosttest'
                    for host in status_hosts)

            )

        self.assertTrue(
            self.try_for_x_seconds(
                function,
                time_to_wait=180,
                cooldown=10,
                exception=AssertionError,
                message="Could not find host in status."
            )
        )

    def test_delete_host(self):
        self.test_create_host()

        TestSeparatedIntegrationSurveil.client.config.hosts.delete(
            'integrationhosttest'
        )

        TestSeparatedIntegrationSurveil.client.config.reload_config()

        def function():
            status_hosts = (TestSeparatedIntegrationSurveil.
                            client.status.hosts.list())
            self.assertFalse(
                any(host['host_name'].decode() == 'integrationhosttest'
                    for host in status_hosts)
            )

        self.assertTrue(
            self.try_for_x_seconds(
                function,
                time_to_wait=180,
                cooldown=10,
                exception=AssertionError,
                message="Host was not deleted"
            )
        )
